"""Command-line interface."""
import sys

import rich_click as click
from rich import print as rprint

import ncdump_rich.ncdump as ncd
from ncdump_rich import __version__

# click.rich_click.USE_RICH_MARKUP = True
click.rich_click.USE_MARKDOWN = True
click.rich_click.SHOW_ARGUMENTS = True
# click.rich_click.GROUP_ARGUMENTS_OPTIONS = True


@click.command()
@click.version_option(version=__version__)
@click.argument("filename", type=click.Path(exists=True, readable=True), required=True)
@click.option(
    "--long/--short",
    "-l/-s",
    default=False,
    show_default=True,
    type=bool,
    help="Print all obtainable info from the .nc file.",
)
@click.option(
    "--format/--no-format",
    "-f/-F",
    default=True,
    show_default=True,
    type=bool,
    help="Print in formatted text. No formatting is better when the output is stored "
    + "and viewed in an editor.",
)
def main(filename: str, long: bool, format: bool) -> None:
    """Rich NcDump.

    Read in a netCDF file as FILENAME and beautifully print a preview of it using
    [Rich](https://github.com/Textualize/rich).
    """
    if filename.endswith(".nc"):
        ncd.ncdump(filename, long=long, truecolor=format)
    else:
        rprint(
            "[bold]ncdump-rich[/bold] can only read [italic green].nc"
            + f"[/italic green] files, not [italic red].{filename.split('.')[-1]}"
            + "[/italic red]"
        )
        sys.exit()


if __name__ == "__main__":
    main(prog_name="ncdump-rich")  # pragma: no cover



"""Formatted output of a .nc file.

This script takes in as input with the `-i` `--input` flag a `.nc` file
and by default truncates the output if a lot of data exist,
so the most important information is presented.

The flag `-l` `--long` will override the truncation and print a long
output with all information contained in the .nc file.
"""
import pprint
import textwrap

import netCDF4  # type: ignore
from rich.console import Console


def ncdump(src_path: str, long: bool = False, truecolor: bool = True) -> None:
    """Output dimensions, variables and their attribute information.

    The information is similar to that of NCAR's ncdump utility.
    ncdump requires a valid instance of Dataset.

    Inspired by: http://schubert.atmos.colostate.edu/~cslocum/netcdf_example.html

    Parameters
    ----------
    src_path: str
        Path to a .nc file
    long: bool (default: False)
        Print all details found in the .nc file
    truecolor: bool (default: True)
        Print with colours and stylised font
    verb: bool (default: True)
        Whether or not nc_attrs, nc_dims, and nc_vars are printed
    """
    nc_file = netCDF4.Dataset(src_path, "r")
    if truecolor:
        console = Console(force_terminal=True, color_system="truecolor", tab_size=4)
    else:
        console = Console(tab_size=4)
    width = console.size.width
    cprint = console.print

    def print_ncattr(key: str) -> None:
        """Print the NetCDF file attributes for a given key.

        Parameters
        ----------
        key: str
            A valid netCDF4.Dataset.variables key
        """
        try:
            cprint(
                "\t\t[italic white]type:[/italic white]",
                repr(nc_file.variables[key].dtype),
            )
            for ncattr in nc_file.variables[key].ncattrs():
                cprint(
                    "\t\t[italic white]%s:[/italic white]" % ncattr,
                    repr(nc_file.variables[key].getncattr(ncattr)),
                )
        except KeyError:
            s1 = "\t\t[italic white]WARNING: [/italic white][red]"
            s2 = f"{key}[/red] does not contain variable attributes"
            cprint(s1 + s2)

    # Print file properties
    cprint("[bold white]NetCDF Properties:[/bold white]")
    for key, prop in {
        "File format": "file_format",
        "Disk format": "disk_format",
    }.items():
        cprint(f"\t[italic white]{key}[/italic white]: {getattr(nc_file, prop)}")

    # NetCDF global attributes
    nc_attrs = nc_file.ncattrs()
    cprint("[bold white]NetCDF Global Attributes:[/bold white]")
    for nc_attr in nc_attrs:
        if any(
            len(line) > width - 9
            for line in str(nc_file.getncattr(nc_attr)).splitlines()
        ):
            lineend = (
                " [italic white dim]4 spaces = new line; 8 spaces = line wrap[/italic"
                " white dim]\n\t\t"
            )
        else:
            lineend = " "
        try:
            if repr(nc_file.getncattr(nc_attr)[0]) != repr("\n"):
                cprint(
                    f"\t[italic white]{nc_attr}:[/italic white]{lineend}"
                    + "\n\t\t".join(
                        [
                            textwrap.fill(
                                " ".join(line.split()),
                                width=width - 9,
                                tabsize=4,
                                break_long_words=False,
                                replace_whitespace=False,
                                subsequent_indent="\t\t\t",
                            )
                            for line in str(nc_file.getncattr(nc_attr)).splitlines()
                        ]
                    ),
                )
            else:
                cprint(
                    f"\t[italic white]{nc_attr}:[/italic white]{lineend}"
                    + "\n\t\t".join(
                        [
                            textwrap.fill(
                                " ".join(line.split()),
                                width=width - 9,
                                tabsize=4,
                                break_long_words=False,
                                replace_whitespace=False,
                                subsequent_indent="\t\t\t",
                            )
                            for line in str(nc_file.getncattr(nc_attr)).splitlines()
                        ]
                    ),
                )
        except IndexError:
            cprint("\t[italic white]%s:[/italic white] [red]empty[/red]" % nc_attr)
    nc_dims = list(nc_file.dimensions)
    cprint("[bold white]NetCDF Dimension Information:[/bold white]")
    for dim in nc_dims:
        cprint("\t[italic white]Name:[/italic white]", dim)
        cprint("\t\t[italic white]size:[/italic white]", len(nc_file.dimensions[dim]))
        print_ncattr(dim)

    # Variable information.
    nc_vars = list(nc_file.variables)  # list of nc variables
    cprint("[bold white]NetCDF Variable Information:[/bold white]")
    if long:
        for var in nc_vars:
            if var not in nc_dims:
                cprint("\t[italic white]Name:[/italic white]", var)
                cprint(
                    "\t\t[italic white]dimensions:[/italic white]",
                    repr(nc_file.variables[var].dimensions),
                )
                cprint(
                    "\t\t[italic white]size:[/italic white]",
                    nc_file.variables[var].size,
                )
                print_ncattr(var)
    else:
        if len(nc_vars) > 20:
            cprint("\t[italic white]Number of variables: [/italic white]", len(nc_vars))
            cprint("\t[italic white]Variables list: [/italic white]")
            pp = pprint.PrettyPrinter(width=width - 9, compact=True)
            cprint(textwrap.indent(pp.pformat(nc_vars), "\t\t"))
        else:
            for var in nc_vars:
                if var not in nc_dims:
                    cprint("\t[italic white]Name:[/italic white]", var)
                    cprint(
                        "\t\t[italic white]dimensions:[/italic white]",
                        repr(nc_file.variables[var].dimensions),
                    )
                    cprint(
                        "\t\t[italic white]size:[/italic white]",
                        nc_file.variables[var].size,
                    )
                    print_ncattr(var)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ncdump_rich']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0',
 'importlib-metadata>=4.12,<7.0',
 'netCDF4>=1.5.7,<2.0.0',
 'rich-click>=1.5.2,<2.0.0',
 'rich>=10.7,<14.0']

entry_points = \
{'console_scripts': ['ncdump-rich = ncdump_rich.__main__:main']}

setup_kwargs = {
    'name': 'ncdump-rich',
    'version': '0.3.4',
    'description': 'Rich NcDump',
    'long_description': 'Rich NcDump\n===========\n\n|PyPI| |PyPI Downloads| |Status| |Python Version|\n|License| |Read the Docs| |Tests| |Codecov|\n|pre-commit| |Black|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/ncdump-rich.svg\n   :target: https://pypi.org/project/ncdump-rich/\n   :alt: PyPI\n.. |PyPI Downloads| image:: https://img.shields.io/pypi/dm/ncdump-rich.svg\n   :target: https://pypi.org/project/ncdump-rich/\n   :alt: PyPI Downloads\n.. |Status| image:: https://img.shields.io/pypi/status/ncdump-rich.svg\n   :target: https://pypi.org/project/ncdump-rich/\n   :alt: Status\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/ncdump-rich\n   :target: https://pypi.org/project/ncdump-rich\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/ncdump-rich\n   :target: https://opensource.org/licenses/GPL-3.0\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/ncdump-rich/latest.svg?label=Read%20the%20Docs\n   :target: https://ncdump-rich.readthedocs.io/\n   :alt: Read the documentation at https://ncdump-rich.readthedocs.io/\n.. |Tests| image:: https://github.com/engeir/ncdump-rich/workflows/Tests/badge.svg\n   :target: https://github.com/engeir/ncdump-rich/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://img.shields.io/codecov/c/gh/engeir/ncdump-rich?label=codecov&logo=codecov\n   :target: https://codecov.io/gh/engeir/ncdump-rich\n   :alt: Codecov\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\n\nFeatures\n--------\n\nThis project provides an easy way of previewing ``netCDF`` files with nicely\nformatted text in your terminal. The information extracted from the ``.nc``\nfiles are obtained in a similar way to `this example`_, with some\nmodifications. The source code used on the website can be downloaded as\n``netcdf_example.py`` with:\n\n.. code:: console\n\n   $ curl -O http://schubert.atmos.colostate.edu/~cslocum/code/netcdf_example.py\n\nTo make the output more readable it is formatted using the python library rich_.\n\n\nRequirements\n------------\n\nThe project depends on the python packages ``click``, ``netCDF4`` and ``rich``.\nInstallation via pip_ or pipx_ ensures that all dependencies are installed correctly.\n\n\nInstallation\n------------\n\nYou can install *Rich NcDump* via pip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install ncdump-rich\n\nor perhaps even better via pipx_:\n\n.. code:: console\n\n   $ pipx install ncdump-rich\n\n\nUsage\n-----\n\nPlease see the `Command-line Reference <Usage_>`_ for details.\n\n.. image:: ./demo/latest.png\n   :width: 600\n\nExamples\n^^^^^^^^\n\nUse the program as a previewer for ``.nc`` files, for example through stpv_. `My own\nfork`_ provides additional support for previewing ``.nc`` files using this project.\n\nPreview in lf_\n\n.. image:: ./demo/lf-demo.png\n   :width: 600\n\nSimilarly you can get preview of ``.nc`` files in nnn_ by including an option for the\nextension ``nc`` in the |preview-tui plugin|_.\n\n.. code:: console\n\n   nc) fifo_pager ncdump-rich "$1" ;;\n\nPreview in nnn_\n\n.. image:: ./demo/nnn-demo.png\n   :width: 600\n\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\nDistributed under the terms of the `GPL 3.0 license`_,\n*Rich NcDump* is free and open source software.\n\n\nIssues\n------\n\nIf you encounter any problems,\nplease `file an issue`_ along with a detailed description.\n\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_\'s `Hypermodern Python Cookiecutter`_\ntemplate.\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _GPL 3.0 license: https://opensource.org/licenses/GPL-3.0\n.. _PyPI: https://pypi.org/\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _file an issue: https://github.com/engeir/ncdump-rich/issues\n.. _pip: https://pip.pypa.io/\n.. _pipx: https://github.com/pypa/pipx\n.. _stpv: https://github.com/Naheel-Azawy/stpv\n.. _My own fork: https://github.com/engeir/stpv\n.. _rich: https://rich.readthedocs.io/en/latest/\n.. _this example: http://schubert.atmos.colostate.edu/~cslocum/netcdf_example.html\n.. _nnn: https://github.com/jarun/nnn\n.. _lf: https://github.com/gokcehan/lf\n.. |preview-tui plugin| replace:: ``preview-tui`` plugin\n.. _preview-tui plugin: https://github.com/jarun/nnn/blob/fc00faf7d0f4cd0b4637e719af52100861e8c17a/plugins/preview-tui#L247\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n.. _Usage: https://ncdump-rich.readthedocs.io/en/latest/usage.html\n',
    'author': 'Eirik Enger',
    'author_email': 'eirroleng@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/engeir/ncdump-rich',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.11',
}


setup(**setup_kwargs)
